/*
##################################################################

demoaliasing v0.2

    IGNACIO DIAZ BLANCO, Universidad de Oviedo, 2023

    Versiones

      v0.1: 2017/03/17 (initial version, uploaded in 
                        http://isa.uniovi.es/~idiaz/demos/demo_aliasing_js/)
    
      v0.2: 2023/03/28 (same funcionality, but with a description and added 
                        comments)
    
    Description
    
      Simple educational application to show and explain the 
      concept of aliasing. The application displays a 2D analog 
      signal in the form of a "turning wheel" or complex exponential, 
      as well as a 1D signal in the form of a sinusoid, which 
      represents the vertical component of the 2D signal. 
      Both the analog and discretized versions of the 2D and 1D 
      signals are displayed, with the analog signals shown in white 
      and the discretized signals shown in red.

      -  white: analog signal
      -    red: discretized version of the analog signal
      - slider: sets the frequency of the analog signal in "per unit" 
                (p.u.) times the sample rate.

    
    Credits
      
      This app was created using the p5.js library (https://p5js.org/).

##################################################################
*/

k 	 = 0;
t 	 = 0;
phi  = 0;
phim = 0;

// tamaño del canvas
h = 400
w = 800



// buffer circular con las señales real y muestreada
buffer = []


function setup()
{
  // inicializamos el canvas
  createCanvas(w,h);

  // inicializamos a ceros el buffer
  for(var i=0; i<w*0.75; i++)
  	buffer[i] = 0


  slider = document.getElementById('sliderFrecuencia')
  texto  = document.getElementById('textoFrecuencia')

  slider.value=0.1
  texto.value=0.1

  f = slider.value


}




function draw()
{

  // limpiar el lienzo
  background(0)

  noStroke()

  // tiempo pseudo-analógico (tm muy pequeño, 40 veces menor)
  t = t+0.05;

  // tiempo discreto
  k = floor(t);

  phi += 2*PI*t*f;
  phim+= 2*PI*k*f;

  // coordenadas del punto real
  x = 100*cos(2*PI*t*f+Math.PI/2)
  y = 100*sin(2*PI*t*f+Math.PI/2)

  // coordenadas del punto muestreado
  xm = 100*cos(2*PI*k*f+Math.PI/2)
  ym = 100*sin(2*PI*k*f+Math.PI/2)
  


  // ejes y circunferencia
  stroke(255)
  noFill()
  ellipse(w*0.8,h/2,200,200)
  noStroke()
  rectMode(CENTER)
  fill(100)
  rect(w*0.8,h/2,.5,200)
  rect(w*0.8,h/2,200,.5)

  // dibujamos el punto real y el muestreado
  fill(255);
  ellipse(x + w*0.8, y + h/2, 10, 10);
  fill(255, 0, 0);
  ellipse(xm + w*0.8, ym + h/2, 10, 10);
  

  // ejes
  stroke(100)
  line(w*0.618,h/2-100,w*0.618,h/2+100)
  line(0,h/2,w*0.618,h/2)
  noStroke()


  buffer.splice(0,0,[y,ym])
  buffer.pop()

  noFill()
  stroke(255)
  beginShape()
  buffer.forEach(function(d,i){
  	vertex(w*0.618-i, d[0] + h/2)
  })
  endShape()

  noFill()
  stroke(255,0,0)
  beginShape()
  buffer.forEach(function(d,i){
  	vertex(w*0.618-i, d[1] + h/2);
  })
  endShape()




}


function cambiaTexto()
{
	f = texto.value
	slider.value=f
}

function cambiaSlider()
{
	f = slider.value
	texto.value=f
}